#!/usr/bin/env python
#
# Copyright (c) 2014, Hugo Osvaldo Barrera <hugo@barrera.io>
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

"""
Convert any object into a pretty-printed string. Mainly usable for debugging,
or logging. There should be little use for this in actual production
environments.
"""


import inspect
from StringIO import StringIO

def to_stringIO(obj, indent=2):
    """
    Returns `obj` pretty-printed into a StringIO.

    @param obj
        The object to print recursively.
    @param indent
        Object members will be pretty-printed with that indent level.
    """

    primitive = (int, str, bool, float)

    def is_primitive(thing):
        return isinstance(thing, primitive)

    def str_id(obj):
        return "@" + repr(id(obj))

    def _to_string(obj, indent, current_indent):

        if isinstance(obj, list):
            string.write(current_indent * indent * " ")
            string.write("<list>:\n")
            for item in obj:
                string.write((current_indent + 1) * indent * " ")
                string.write(_to_string(item, indent, current_indent + 1))
        elif isinstance(obj, str):
            string.write("\"")
            string.write(obj)
            string.write("\"\n")
        elif is_primitive(obj):
            string.write(obj)
            string.write("\n")
        else:
            string.write(obj.__class__.__name__ + ":\n")
            current_indent += 1
            for name, value in inspect.getmembers(obj):
                string.write( current_indent * indent * " ")
                if name.startswith("__"):
                    string.write(name + ": <hidden " + str_id(value) + ">\n")
                elif (inspect.ismethod(value)):
                    string.write(name + ": <function " + str_id(value) + ">\n")
                else:
                    string.write(name + ": ")
                    string.write(_to_string(value, indent, current_indent))

    string = StringIO()

    _to_string(obj, indent, 0)
    return string


def to_string(obj, indent=2):
    """
    Returns `obj` pretty-printed into a string.

    @param obj
        The object to print recursively.
    @param indent
        Object members will be pretty-printed with that indent level.
    """
    return to_stringIO(obj, indent).getvalue()


if __name__ == "__main__":
    """
    This is NOT a test unit. This is just some code to quickly test the output.
    """

    class TestClass():

        def __init__(self, parent=True):
            self.one = 1
            self.two = 2
            if parent:
                self.three = TestClass(False)
                self.four = "4"

        def func(self):
            return 0

    t = TestClass()
    print(to_string(t))
    print(to_string([t, 7, t]))
